@@ -21,23 +21,33 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
+#pragma once
 
 #include <stdint.h>
 
 #include <osmocom/core/isdnhdlc.h>
 #include <osmocom/core/linuxlist.h>
 #include <osmocom/core/rate_ctr.h>
+#include <osmocom/core/stat_item.h>
 #include <osmocom/core/timer.h>
 #include <osmocom/vty/command.h>
 
+#include <osmocom/octoi/octoi.h>
 #include <osmocom/e1d/proto.h>
 
+/***********************************************************************
+ * core e1d related data structures
+ ***********************************************************************/
+
 enum e1d_vty_node {
 	E1D_NODE = _LAST_OSMOVTY_NODE + 1,
-	LINE_NODE
+	INTF_NODE,
+	LINE_NODE,
 };
 
 #define line_ctr_add(line, idx, add) rate_ctr_add(rate_ctr_group_get_ctr((line)->ctrs, idx), add)
+#define line_stat_set(line, idx, add) \
+	osmo_stat_item_set(osmo_stat_item_group_get_item((line)->stats, idx), add)
 
 enum e1d_line_ctr {
 	LINE_CTR_LOS,
@@ -47,6 +57,16 @@ enum e1d_line_ctr {
 	LINE_CTR_TX_UNFL,
 	LINE_CTR_RX_REMOTE_E,
 	LINE_CTR_RX_REMOTE_A,
+	LINE_CTR_FRAMES_MUXED_E1T,
+	LINE_CTR_FRAMES_DEMUXED_E1O,
+};
+
+enum e1d_line_stat_item {
+	LINE_GPSDO_STATE,
+	LINE_GPSDO_ANTENNA,
+	LINE_GPSDO_TUNE_COARSE,
+	LINE_GPSDO_TUNE_FINE,
+	LINE_GPSDO_FREQ_EST,
 };
 
 enum e1_ts_mode {
@@ -79,6 +99,7 @@ struct e1_ts {
 		uint8_t *rx_buf;		/* actual buffer storage */
 		unsigned int rx_buf_size;	/* size of 'buf' in bytes */
 		unsigned int rx_buf_used;	/* number of bytes used so far */
+		bool tx_started;		/* tx started */
 	} raw;
 
 	/* Remote end */
@@ -91,6 +112,8 @@ enum e1_line_mode {
 	/* 1 channel group spanning all 31 TS, as used e.g. when using Frame Relay
 	 * or raw HDLC over channelized E1. */
 	E1_LINE_MODE_SUPERCHANNEL,
+	/* E1 forwarding over IP */
+	E1_LINE_MODE_E1OIP,
 };
 
 #define E1L_TS0_RX_CRC4_ERR	0x01
@@ -106,11 +129,13 @@ struct e1_line {
 
 	void *drv_data;
 	struct rate_ctr_group *ctrs;
+	struct osmo_stat_item_group *stats;
 
 	/* timeslots for channelized mode */
 	struct e1_ts ts[32];
 	/* superchannel */
 	struct e1_ts superchan;
+	struct octoi_peer *octoi_peer;
 
 	struct {
 		/*! buffer where we aggregate the E bits each multi-frame */
@@ -146,6 +171,11 @@ struct e1_intf {
 	struct e1_daemon *e1d;
 	uint8_t id;
 
+	struct {
+		char *serial_str;
+	} usb;
+
+	bool vty_created;
 	enum e1_driver drv;
 	void *drv_data;
 
@@ -157,6 +187,7 @@ struct e1_daemon {
 	struct llist_head interfaces;
 };
 
+extern const struct octoi_ops e1d_octoi_ops;
 
 struct e1_line *
 e1_intf_find_line(struct e1_intf *intf, uint8_t id);
@@ -165,11 +196,14 @@ struct e1_line *
 e1_intf_find_line(struct e1_intf *intf, uint8_t id);
 
 struct e1_intf *
-e1_intf_new(struct e1_daemon *e1d, void *drv_data);
+e1_intf_new(struct e1_daemon *e1d, int intf_id, void *drv_data);
 
 struct e1_intf *
 e1d_find_intf(struct e1_daemon *e1d, uint8_t id);
 
+struct e1_intf *
+e1d_find_intf_by_usb_serial(struct e1_daemon *e1d, const char *serial_str);
+
 void
 e1_intf_destroy(struct e1_intf *intf);
 
@@ -178,11 +212,14 @@ struct e1_line *
 e1_intf_find_line(struct e1_intf *intf, uint8_t id);
 
 struct e1_line *
-e1_line_new(struct e1_intf *intf, void *drv_data);
+e1_line_new(struct e1_intf *intf, int line_id, void *drv_data);
 
 void
 e1_line_destroy(struct e1_line *line);
 
+void
+e1_line_active(struct e1_line *line);
+
 int
 e1_line_mux_out(struct e1_line *line, uint8_t *buf, int fts);
 
@@ -200,3 +237,12 @@ e1d_vpair_create(struct e1_daemon *e1d, unsigned int num_lines);
 
 struct e1_intf *
 e1d_vpair_intf_peer(struct e1_intf *intf);
+
+int
+e1oip_line_demux_in(struct e1_line *line, const uint8_t *buf, int ftr);
+
+int
+e1oip_line_mux_out(struct e1_line *line, uint8_t *buf, int fts);
+
+int
+e1d_vty_go_parent(struct vty *vty);
