@@ -23,8 +23,54 @@ enum e1usb_dev_capability {
 	ICE1USB_DEV_CAP_GPSDO,
 };
 
+/***********************************************************************
+ * Control Endpoint / GPS-DO Interface Requests
+ ***********************************************************************/
 
-/* Interface Requests */
+#define ICE1USB_INTF_GET_GPSDO_STATUS	0x10
+#define ICE1USB_INTF_GET_GPSDO_MODE	0x12	/*!< uint8_t */
+#define ICE1USB_INTF_SET_GPSDO_MODE	0x13	/*!< wValue = mode */
+#define ICE1USB_INTF_GET_GPSDO_TUNE	0x14	/*!< data   = struct e1usb_gpsdo_tune */
+#define ICE1USB_INTF_SET_GPSDO_TUNE	0x15	/*!< data   = struct e1usb_gpsdo_tune */
+
+enum ice1usb_gpsdo_mode {
+	ICE1USB_GPSDO_MODE_DISABLED	= 0,
+	ICE1USB_GPSDO_MODE_AUTO		= 1,
+};
+
+enum ice1usb_gpsdo_antenna_state {
+	ICE1USB_GPSDO_ANT_UNKNOWN	= 0,
+	ICE1USB_GPSDO_ANT_OK		= 1,
+	ICE1USB_GPSDO_ANT_OPEN		= 2,
+	ICE1USB_GPSDO_ANT_SHORT		= 3,
+};
+
+enum ice1usb_gpsdo_state {
+	ICE1USB_GPSDO_STATE_DISABLED	= 0,
+	ICE1USB_GPSDO_STATE_CALIBRATE	= 1,
+	ICE1USB_GPSDO_STATE_HOLD_OVER	= 2,
+	ICE1USB_GPSDO_STATE_TUNE_COARSE	= 3,
+	ICE1USB_GPSDO_STATE_TUNE_FINE	= 4,
+};
+
+struct e1usb_gpsdo_tune {
+	uint16_t coarse;
+	uint16_t fine;
+} __attribute__((packed));
+
+struct e1usb_gpsdo_status {
+	uint8_t state;
+	uint8_t antenna_state;		/*!< Antenna state */
+	uint8_t valid_fix;		/*!< Valid GPS Fix (0/1) */
+	uint8_t mode;			/*!< Current configured operating mode */
+	struct e1usb_gpsdo_tune tune;	/*!< Current VCXO tuning values */
+	uint32_t freq_est;		/*!< Latest frequency estimate measurement */
+} __attribute__((packed));
+
+
+/***********************************************************************
+ * Control Endpoint / E1 Interface Requests
+ ***********************************************************************/
 
 /*! returns a bit-mask of optional device capabilities (see enum e1usb_intf_capability) */
 #define ICE1USB_INTF_GET_CAPABILITIES	0x01
@@ -32,6 +78,7 @@ enum e1usb_dev_capability {
 #define ICE1USB_INTF_GET_TX_CFG		0x03	/*!< struct ice1usb_tx_config */
 #define ICE1USB_INTF_SET_RX_CFG		0x04	/*!< struct ice1usb_rx_config */
 #define ICE1USB_INTF_GET_RX_CFG		0x05	/*!< struct ice1usb_rx_config */
+#define ICE1USB_INTF_GET_ERRORS		0x06	/*!< struct ice1usb_irq_err */
 
 //enum e1usb_intf_capability { };
 
@@ -85,9 +132,10 @@ enum ice1usb_irq_type {
 	ICE1USB_IRQ_T_ERRCNT		= 1,
 };
 
-/* Ensue ro keep those in sync with e1.h */
+/* Ensue to keep those in sync with e1.h */
 #define ICE1USB_ERR_F_ALIGN_ERR	0x01
 #define ICE1USB_ERR_F_LOS	0x02
+#define ICE1USB_ERR_F_RAI	0x04
 
 struct ice1usb_irq_err {
 	/* 16-bit little-endian counters */
